//importing the required utilities 
import {select,
        csv,
        scaleLinear,
        max, 
        scaleBand,
        axisLeft,
        axisBottom
       } from 'd3';
//defining the constants 
const svg = select('svg');
const xlabel = 'Percentage';

const width = +svg.attr('width');
const height = +svg.attr('height');
   //defining the tool tip attributes
      const tooltip = d3.select("body")
        .append("div")
        .attr("class","d3-tooltip")
        .style("position", "absolute")
        .style("z-index", "10")
        .style("visibility", "hidden")
        .style("padding", "15px")
        .style("background", "rgba(0,0,0,0.6)")
        .style("border-radius", "5px")
        .style("color", "#fff")
        .text("a simple tooltip");

//rendering the bar chart
const render = data => {
  const xValue = d => d.percentage;
  const yValue = d=>d.country;
  const margin = {top:20,right:20,left:100,bottom:20};
  const innerWidth = width - margin.left - margin.right
  const innerHeight  = height - margin.top - margin.bottom
  const xScale = scaleLinear()
    .domain([0,max(data,xValue)])
    .range([0,innerWidth]);
  
  const yScale =scaleBand()
  	.domain(data.map(yValue))
  	.range([0,innerHeight])
  	.padding(0.1);
  const yAxis = axisLeft(yScale);
  
  const g = svg.append('g')
  	.attr('transform',`translate(${margin.left},${margin.top})`)
  
  yAxis(g.append('g'));
  
  g.append('g').call(axisBottom(xScale))
  	.attr('transform',`translate(0,${innerHeight})`);
//displaying title 
  g.append('text')
  	.attr('class','title')
  	.attr('y',0)
  	.attr('x',150)
  	.text("List of Countries by percentage of Internet Users");
  
  g.selectAll('rect').data(data)
  .enter().append('rect')
  	.attr('y',d => yScale(yValue(d)))
    .attr('width',d => xScale(xValue(d)))
  	.attr('height',yScale.bandwidth())
    .on("mouseover", function(event,d) {
      tooltip.html(`${d.country} Internet Users Rank: ${d.rank}`).style("visibility", "visible");
      d3.select(this)
        .attr("fill", shadeColor(bar_color, -15));
    })
    .on("mousemove", function(){
      tooltip
        .style("top", (event.pageY-10)+"px")
        .style("left",(event.pageX+10)+"px");
    })
    .on("mouseout", function() {
      tooltip.html(``).style("visibility", "hidden");
      d3.select(this).attr("fill", bar_color);
    });
};
//importing published data set
const dataUrl="https://gist.githubusercontent.com/DeepikaGoud08/be7615ad94e42008be267b807881a812/raw/e77f65555c306136f5ec787d6c7685e294d26000/internetaccounts.csv"
csv(dataUrl).then(data => {
  data.forEach( d => {
    d.percentage  = +d.percentage;
  });
  render(data);
});